from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
from processor import GraphProcessor
import tkinter as tk
import tkinter.ttk as ttk
import pandas as pd
from PIL import Image, ImageTk
import matplotlib

matplotlib.use('TkAgg')


class Widget:

    def __init__(self, root, bg, label='', df=''):
        self.__fm_img = None
        self.__text_label = None
        self.__status_listbox = False
        self.__status_cb_menu = False
        self.__cb_menu = None
        self.__treeview = None
        self.__status_treeview = True
        self.__another_nutrient_value_list = []
        self.__lbf_another_nutrient = None
        self.__graph_type = "Bar Graph"
        self.__nutrient = "energy (kcal/kJ)"
        self.__another_nutrient = "water (g)"
        self.__fm_cb = None
        self._BG_COLOR = '#B9EDDD'

        self.__nutrient_value_list = []
        self.__menu = []
        self.graph_list = ["Bar Graph", "Histogram Graph", "Box Plot", "Scatter Plot", "Pie Chart"]

        self.__root = root
        self.__label = label
        self.__bg = bg
        self.__df = df

        self.__fm2 = tk.Frame(self.__root, bd=2, highlightbackground="black", highlightthickness=3)
        self.__fm2.grid(row=2, column=1, padx=30, pady=20)

    def create_topic_label(self):
        topic_label = tk.Label(
            text="Fruity & Veggies",
            font=('arial', 90, 'bold'),
            width=28,
            background='#EEEEEE',
            foreground='#577D86')
        topic_label.grid(row=0, column=0, columnspan=2, sticky='nsew')

    def image_homepage(self):
        self.__fm_img = tk.Frame(width=600, height=400)
        self.__fm_img.grid(row=1, column=0, columnspan=2, pady=60)

        self.bg_image = ImageTk.PhotoImage(Image.open("fruitveg.jpg"))
        label_img = tk.Label(self.__fm_img, image=self.bg_image)
        label_img.grid(row=0, column=0)

    def create_label(self):
        self.__text_label = tk.Label(self.__root,
                                     text=self.__label,
                                     font=('arial', 50, 'bold'),
                                     bg=self.__bg,
                                     width=49)
        self.__text_label.grid(row=1, column=0, columnspan=2, pady=10)

    def create_listbox(self):
        self.__status_listbox = True
        self.__nutrient_value_list = []
        self.__menu = []

        self.__fm = tk.Frame(self.__root, bd=2, highlightbackground="black", highlightthickness=3)
        self.__fm.grid(row=2, column=0, padx=30, pady=20)

        # Create Listbox and Scroll bar
        self.__scrollbar = tk.Scrollbar(self.__fm)
        self.__scrollbar.pack(side="right", fill="y")

        self.__listbox = tk.Listbox(self.__fm,
                                    height=13,
                                    width=40,
                                    font=("Arial", 20),
                                    yscrollcommand=self.__scrollbar.set)
        self.__scrollbar.config(command=self.__listbox.yview)
        self.__scrollbar.pack(side="right", fill="y")
        self.__listbox.pack()

        for item in self.__df['name']:
            self.__listbox.insert("end", item)

        self.__listbox.bind("<<ListboxSelect>>", self.text_display)

        self.__listbox.bind("<<ListboxSelect>>")  # test
        self.__listbox.focus_set()  # auto display listbox
        self.__listbox.pack()

    def create_treeview(self):
        self.__scrollbar2 = tk.Scrollbar(self.__fm2)
        self.__scrollbar2.pack(side="right", fill="y")

        self.__treeview = ttk.Treeview(self.__fm2,
                                     columns=("name", "nutrient"),
                                     show="headings",
                                     height=16,
                                     yscrollcommand=self.__scrollbar2.set,
                                     )

        self.__scrollbar2.config(command=self.__treeview.yview)
        self.__scrollbar2.pack(side="right", fill="y")

        self.__treeview.heading("name", text="Name")
        self.__treeview.heading("nutrient", text=self.__nutrient)

        self.__treeview.column("name", width=500)
        self.__treeview.column("nutrient", width=100)

        self.__treeview.pack()

    def create_clear_button(self):
        self.__fm_bt1 = tk.Frame(self.__root, bg=self.__bg)
        self.__fm_bt1.grid(row=3, column=1, pady=20)

        self.__clear_bt1 = tk.Button(self.__fm_bt1, text="Clear", font=('arial', 30, 'bold'), command=self.clear_item)
        self.__clear_bt1.grid(row=0, column=0, padx=30)

    def create_select_button(self):
        select_bt1 = tk.Button(self.__fm_bt1,
                               text="Select All",
                               font=('arial', 30, 'bold'),
                               command=self.select_item)
        select_bt1.grid(row=0, column=1, padx=30)

    def create_lbf_nutrient(self):
        self.__nutrient_name_list = list(self.__df.columns[1:])
        self.__fm_cb = tk.Frame(self.__root, bg=self.__bg)

        self.__text_labelFrame1 = tk.LabelFrame(self.__fm_cb, text="Select Nutrient Type", bg=self.__bg)

        self.__cb_type = ttk.Combobox(self.__text_labelFrame1, values=self.__nutrient_name_list)
        self.__cb_type.current(0)
        self.__cb_type.bind('<Key>', 'break')
        self.__cb_type.bind('<<ComboboxSelected>>', self.cb_selected_nutrient)
        self.__cb_type.pack()

        self.__text_labelFrame1.grid(row=0, column=1, padx=15)

        self.__fm_cb.grid(row=3, column=0)

    def create_lbf_chart(self):
        self.__lbf_chart = tk.LabelFrame(self.__fm_cb, text="Select Chart Type", bg=self.__bg)

        self.__cb_type2 = ttk.Combobox(self.__lbf_chart, values=self.graph_list)
        self.__cb_type2.current(0)
        self.__cb_type2.bind('<Key>', 'break')  # ไม่ให้พิมใน combobox
        self.__cb_type2.bind('<<ComboboxSelected>>', self.cb_selected_chart)
        self.__cb_type2.pack()

        self.__lbf_chart.grid(row=0, column=0, padx=15)

    def create_lbf_chart2(self):
        self.__lbf_chart2 = tk.LabelFrame(self.__root, text="Select Chart Type", bg=self.__bg)

        self.__cb_type2 = ttk.Combobox(self.__lbf_chart, values=self.graph_list)
        self.__cb_type2.current(0)
        self.__cb_type2.bind('<Key>', 'break')  # ไม่ให้พิมใน combobox
        self.__cb_type2.bind('<<ComboboxSelected>>', self.cb_selected_chart)
        self.__cb_type2.pack()

        self.__lbf_chart2.grid(row=1, column=1, padx=15)

    def create_lbf_descriptive_statistics(self, nutrient, value_df=None, min=None, max=None, mean=None, graph=None):
        self.__lbf_stat = tk.LabelFrame(self.__root, text=f"Descriptive Statistics {nutrient}", bg=self.__bg)
        if graph == "Pie Chart":
            min = round(min, 5)
            max = round(max, 5)
            mean = round(mean, 5)

            var_min = tk.DoubleVar()
            var_min.set(min)

            var_max = tk.DoubleVar()
            var_max.set(max)

            var_mean = tk.DoubleVar()
            var_mean.set(mean)

            min_lbf = tk.LabelFrame(self.__lbf_stat, text="Min", bg=self.__bg)
            min_text = tk.Entry(min_lbf, textvariable=var_min, state='disabled', width=15)
            min_text.pack()
            min_lbf.grid(row=0, column=0, padx=10, pady=10)

            max_lbf = tk.LabelFrame(self.__lbf_stat, text="Max", bg=self.__bg)
            max_text = tk.Entry(max_lbf, textvariable=var_max, state='disabled', width=15)
            max_text.pack()
            max_lbf.grid(row=0, column=1, padx=10, pady=10)

            mean_lbf = tk.LabelFrame(self.__lbf_stat, text="Mean", bg=self.__bg)
            mean_text = tk.Entry(mean_lbf, textvariable=var_mean, state='disabled', width=15)
            mean_text.pack()
            mean_lbf.grid(row=0, column=2, padx=10, pady=10)

        else:
            stats = value_df.describe()

            min = round(stats.loc['min'], 5)
            max = round(stats.loc['max'], 5)
            mean = round(stats.loc['mean'], 5)

            total = round(value_df.sum(), 5)
            std = round(stats.loc['std'], 5)
            var = round(std ** 2, 5)

            # corr = round(value_df.corr(), 3)
            # corr = 99999
            q1 = round(stats.loc['25%'], 5)
            q3 = round(stats.loc['75%'], 5)

            iqr = round(q3 - q1, 5)
            out_min = round(q1 - 1.5 * iqr, 5)
            out_max = round(q3 + 1.5 * iqr, 5)

            var_min = tk.DoubleVar()
            var_min.set(min)

            var_max = tk.DoubleVar()
            var_max.set(max)

            var_mean = tk.DoubleVar()
            var_mean.set(mean)

            var_total = tk.DoubleVar()
            var_total.set(total)

            var_std = tk.DoubleVar()
            var_std.set(std)

            var_var = tk.DoubleVar()
            var_var.set(var)

            # var_corr = tk.DoubleVar()
            # var_corr.set(corr)

            var_q1 = tk.DoubleVar()
            var_q1.set(q1)

            var_q3 = tk.DoubleVar()
            var_q3.set(q3)

            var_iqr = tk.DoubleVar()
            var_iqr.set(iqr)

            var_out_min = tk.DoubleVar()
            var_out_min.set(out_min)

            var_out_max = tk.DoubleVar()
            var_out_max.set(out_max)

            min_lbf = tk.LabelFrame(self.__lbf_stat, text="Min", bg=self.__bg)
            min_text = tk.Entry(min_lbf, textvariable=var_min, state='disabled', width=15)
            min_text.pack()
            min_lbf.grid(row=0, column=0, padx=20)

            max_lbf = tk.LabelFrame(self.__lbf_stat, text="Max", bg=self.__bg)
            max_text = tk.Entry(max_lbf, textvariable=var_max, state='disabled', width=15)
            max_text.pack()
            max_lbf.grid(row=0, column=1, padx=20)

            mean_lbf = tk.LabelFrame(self.__lbf_stat, text="Mean", bg=self.__bg)
            mean_text = tk.Entry(mean_lbf, textvariable=var_mean, state='disabled', width=15)
            mean_text.pack()
            mean_lbf.grid(row=0, column=2, pady=10)

            total_lbf = tk.LabelFrame(self.__lbf_stat, text="Total", bg=self.__bg)
            total_text = tk.Entry(total_lbf, textvariable=var_total, state='disabled', width=15)
            total_text.pack()
            total_lbf.grid(row=1, column=0, padx=20, pady=10)

            std_lbf = tk.LabelFrame(self.__lbf_stat, text="Standard Deviation", bg=self.__bg)
            std_text = tk.Entry(std_lbf, textvariable=var_std, state='disabled', width=15)
            std_text.pack()
            std_lbf.grid(row=1, column=1, padx=20)

            var_lbf = tk.LabelFrame(self.__lbf_stat, text="Variance", bg=self.__bg)
            var_text = tk.Entry(var_lbf, textvariable=var_var, state='disabled', width=15)
            var_text.pack()
            var_lbf.grid(row=1, column=2, padx=20)

            # corr_lbf = tk.LabelFrame(self.__lbf_stat, text="Correlation", bg=self.__bg)
            # corr_text = tk.Entry(corr_lbf, textvariable=var_corr, state='disabled', width=15)
            # corr_text.pack()
            # corr_lbf.grid(row=2, column=0, padx=20, pady=10)

            q1_lbf = tk.LabelFrame(self.__lbf_stat, text="Quartile 1", bg=self.__bg)
            q1_text = tk.Entry(q1_lbf, textvariable=var_q1, state='disabled', width=15)
            q1_text.pack()
            q1_lbf.grid(row=2, column=0, padx=20)

            q3_lbf = tk.LabelFrame(self.__lbf_stat, text="Quartile 3", bg=self.__bg)
            q3_text = tk.Entry(q3_lbf, textvariable=var_q3, state='disabled', width=15)
            q3_text.pack()
            q3_lbf.grid(row=2, column=1, padx=20)

            iqr_lbf = tk.LabelFrame(self.__lbf_stat, text="Inter Quartile Range (IQR)", bg=self.__bg)
            iqr_text = tk.Entry(iqr_lbf, textvariable=var_iqr, state='disabled', width=15)
            iqr_text.pack()
            iqr_lbf.grid(row=2, column=2, padx=20, pady=10)

            out_min_lbf = tk.LabelFrame(self.__lbf_stat, text="Outlier Min", bg=self.__bg)
            out_min_text = tk.Entry(out_min_lbf, textvariable=var_out_min, state='disabled', width=15)
            out_min_text.pack()
            out_min_lbf.grid(row=3, column=0, padx=20)

            out_max_lbf = tk.LabelFrame(self.__lbf_stat, text="Outlier Max", bg=self.__bg)
            out_max_text = tk.Entry(out_max_lbf, textvariable=var_out_max, state='disabled', width=15)
            out_max_text.pack()
            out_max_lbf.grid(row=3, column=1, padx=20)

        self.__lbf_stat.grid(row=2, column=1)

    def create_cb_menu(self):
        self.__status_cb_menu = True
        self.__cb_menu = ttk.Combobox(self.__fm2, values=list(self.__df['name']), width=50, font=("Arial", 20, "bold"))
        self.__cb_menu.set(f'Choose a {self.__label}')
        self.__cb_menu.bind('<Key>', 'break')
        self.__cb_menu.bind('<<ComboboxSelected>>', self.cb_selected_menu)

        self.__cb_menu.grid(sticky='news')

    def page1(self):
        self.create_topic_label()
        self.image_homepage()

    def page2(self):
        self.create_label()

        self.create_listbox()

        self.create_treeview()

        self.create_clear_button()

        self.create_select_button()

        self.create_lbf_nutrient()

        self.create_lbf_chart()

    def page3(self):

        new_df, nutrient, graph, nutrient2 = self.get_df_treeview()

        self.__graph_maker = GraphProcessor(df=new_df,
                                   nutrient=nutrient,
                                   title=self.__label)
        if graph == "Pie Chart":
            x = [float(i) for i in new_df['value'].tolist()]
            self.create_lbf_descriptive_statistics(min=min(x),
                                                   max=max(x),
                                                   mean=sum(x) / len(x),
                                                   nutrient=nutrient,
                                                   graph=graph)
        else:
            self.create_lbf_descriptive_statistics(value_df=new_df[self.__nutrient],
                                                   nutrient=nutrient,
                                                   graph=graph)

        if graph == "Bar Graph":
            self.__graph_maker.bar_processor()
        elif graph == "Histogram Graph":
            self.__graph_maker.hist_processor()
        elif graph == "Pie Chart":
            self.__graph_maker.pie_processor()
        elif graph == "Scatter Plot":
            self.__graph_maker.scatter_processor(nutrient2)
        elif graph == "Box Plot":
            self.__graph_maker.boxplot_processor()

        fig_hist = Figure()
        self.ax_hist = fig_hist.add_subplot()

        self.__fig_canvas = FigureCanvasTkAgg(self.__graph_maker.fig, master=self.__root)
        self.__fig_canvas.get_tk_widget().grid(row=1, column=0, rowspan=4,
                                               sticky="news", padx=10, pady=10)

    def text_display(self, event):
        menu = self.__listbox.get(self.__listbox.curselection())
        for index, item in enumerate(self.__df['name']):
            if menu == item:
                result = self.__df[self.__nutrient][index]
                if self.__nutrient == "energy (kcal/kJ)":
                    number = self.__df[self.__nutrient][index]
                    kcal, kj = number.split("/")
                    result = round(int(kj) / int(kcal), 3)

                if self.__graph_type == "Scatter Plot":
                    result2 = self.__df[self.__another_nutrient][index]
                    self.__treeview.insert(parent='', index=tk.END, values=(menu, result, result2))
                    self.__nutrient_value_list.append(result2)
                    self.__another_nutrient_value_list.append(result)
                    self.__menu.append(menu)

                else:
                    self.__treeview.insert(parent='', index=tk.END, values=(menu, result))
                    self.__nutrient_value_list.append(result)
                    self.__menu.append(menu)

                break

        selected_item = self.__listbox.curselection()
        if selected_item:
            self.__listbox.delete(selected_item)

    def clear_item(self):
        self.__clear_bt1.bell()
        self.__nutrient_value_list = []
        self.__menu = []
        self.__listbox.delete(0, tk.END)
        for item in self.__df['name']:
            self.__listbox.insert("end", item)
        self.__treeview.delete(*self.__treeview.get_children())

    def select_item(self):
        result = ''
        self.__listbox.bell()
        self.__nutrient_value_list = []
        self.__menu = []
        self.__listbox.delete(0, tk.END)
        self.__treeview.delete(*self.__treeview.get_children())

        for index, item in enumerate(self.__df['name']):

            if self.__nutrient == "energy (kcal/kJ)":
                number = self.__df[self.__nutrient][index]
                kcal, kj = number.split("/")
                result = round(int(kj) / int(kcal), 3)
            else:
                result = float(self.__df[self.__nutrient][index])

            if self.__graph_type == "Scatter Plot":
                result2 = self.__df[self.__another_nutrient][index]
                self.__treeview.insert(parent='', index=tk.END, values=(item, result, result2))
                self.__nutrient_value_list.append(result2)
                self.__another_nutrient_value_list.append(result)
                self.__menu.append(item)

            else:
                self.__treeview.insert(parent='', index=tk.END, values=(item, result))
                self.__menu.append(item)
                self.__nutrient_value_list.append(result)

        self.__listbox.selection_set(0, tk.END)

    def cb_selected_nutrient(self, event):
        self.__nutrient = self.__cb_type.get()
        self.__treeview.heading("nutrient", text=self.__nutrient)
        if self.__status_treeview:
            self.__treeview.delete(*self.__treeview.get_children())

    def cb_selected_chart(self, event):
        self.__graph_type = self.__cb_type2.get()

        if self.__graph_type == "Scatter Plot":
            if self.__status_cb_menu:
                self.__cb_menu.destroy()
                self.create_treeview()
                self.create_listbox()

                self.create_clear_button()
                self.create_select_button()
                self.__status_listbox = True
                self.__status_cb_menu = False

            remove_choose_nutrient = [x for x in self.__nutrient_name_list if x != self.__nutrient]
            self.__lbf_another_nutrient = tk.LabelFrame(self.__fm_cb, text="Select Another Nutrient Type", bg=self.__bg)
            self.__cb_type_remove = ttk.Combobox(self.__lbf_another_nutrient, values=remove_choose_nutrient)
            self.__cb_type_remove.current(0)
            self.__cb_type_remove.bind('<Key>', 'break')
            self.__cb_type_remove.bind('<<ComboboxSelected>>', self.cb_selected_nutrient_another)
            self.__cb_type_remove.pack()

            self.__lbf_another_nutrient.grid(row=0, column=2, padx=15)

            self.__treeview.config(columns=('name', 'nutrient', 'another_nutrient'))
            self.__treeview.heading("name", text="Name")
            self.__treeview.heading("nutrient", text=self.__nutrient)
            self.__treeview.heading("another_nutrient", text=self.__another_nutrient)

        elif self.__graph_type == "Pie Chart":
            self.__fm_bt1.destroy()
            self.__fm.destroy()
            self.__status_treeview = False
            self.__status_cb_menu = False
            if self.__lbf_another_nutrient is not None:
                self.__lbf_another_nutrient.destroy()
            self.__cb_type.config(values=['Macronutrients', 'Minerals', 'Vitamins'])
            self.__cb_type.set("Choose a Nutrient")

            self.__treeview.destroy()
            self.__scrollbar2.destroy()

            self.create_cb_menu()

            self.__fm_cb.config(pady=10)

        else:
            if self.__lbf_another_nutrient is not None:
                self.__lbf_another_nutrient.destroy()

            if self.__status_cb_menu:
                self.__cb_menu.destroy()
                self.create_treeview()

                self.create_clear_button()
                self.create_select_button()
                self.create_listbox()
                self.__status_listbox = True
                self.__status_cb_menu = False

            self.__cb_type.config(values=self.__nutrient_name_list)
            self.__cb_type.current(0)

    def cb_selected_menu(self, event):
        menu = self.__cb_menu.get()
        menu_df = self.__df[self.__df['name'] == menu]
        _list = []
        self.__pie_df = {'label': [], 'value': [], 'name': menu}
        if self.__nutrient == "Macronutrients":
            _list = ['protein (g)', 'carbohydrates (g)', 'sugars (g)']

        elif self.__nutrient == "Minerals":
            _list = ['calcium (mg)', 'iron (mg)', 'magnessium (mg)', 'phosphorus (mg)',
                     'potassium (mg)', 'sodium (g)']

        elif self.__nutrient == "Vitamins":
            _list = ['vitamin A (IU)', 'vitamin C (mg)', 'vitamin B1 (mg)', 'vitamin B2 (mg)',
                     'vitamin B3 (mg)', 'vitamin B5 (mg)', 'vitamin B6 (mg)', 'vitamin E (mg)']

        self.__pie_df['value'] = menu_df[_list].values.tolist()[0]
        self.__pie_df['label'] = _list

    def cb_selected_nutrient_another(self, event):
        self.__another_nutrient = self.__cb_type_remove.get()
        self.__treeview.heading("another_nutrient", text=self.__another_nutrient)
        if self.__status_treeview:
            self.__treeview.delete(*self.__treeview.get_children())

    def delete_widget_page2(self):
        self.__text_label.destroy()
        self.__listbox.destroy()
        self.__fm2.destroy()
        self.__fm_bt1.destroy()
        self.__fm_cb.destroy()
        self.__fm.destroy()

    def get_df_treeview(self):
        graph_df_to_process = None
        if self.__graph_type == "Scatter Plot":
            graph_df_to_process = pd.DataFrame(
                {'name': self.__menu,
                 self.__nutrient: self.__nutrient_value_list,
                 self.__another_nutrient: self.__another_nutrient_value_list})
            return graph_df_to_process, self.__nutrient, self.__graph_type, self.__another_nutrient

        elif self.__graph_type == "Pie Chart":
            graph_df_to_process = pd.DataFrame(self.__pie_df)

        else:
            graph_df_to_process = pd.DataFrame({'name': self.__menu, self.__nutrient: self.__nutrient_value_list})

        return graph_df_to_process, self.__nutrient, self.__graph_type, None

    def destroy_page1(self):
        self.__fm_img.destroy()

    def destroy_widget_page2(self):
        self.__text_label.destroy()
        self.__fm_cb.destroy()
        self.__fm.destroy()
        self.__fm_bt1.destroy()
        self.__fm2.destroy()

    def destroy_widget_page3(self):
        self.__fig_canvas.get_tk_widget().destroy()
        self.__lbf_stat.destroy()

    def check_element_treeview(self):
        if len(self.__treeview.get_children()) != 0:
            return True
        return False

    def get_graph_type(self):
        return self.__graph_type

    def check_piechart(self):
        if (self.__nutrient in ['Macronutrients', 'Minerals', 'Vitamins']) and (
                self.__cb_menu.get() in self.__df['name'].tolist()):
            return True
        return False
